/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef DOCUMENTS_MAINPAGE_H_
#define DOCUMENTS_MAINPAGE_H_

// This file defines what appears on the Main Page of the documentation
// generated by doxygen. The file contains no code, and does not appear
// in any cpp include statement.
//
/*!
 * \mainpage Object-Oriented Prediction System
 *
 * \section IntroSection History
 *
 * Data assimilation had historically been an integral part of the IFS. 
 * While such approach seemed natural at the time when 4D-Var was being 
 * developed, the system quickly outgrew its original design and became 
 * difficult to maintain and modify. As a consequence, it was difficult 
 * to prototype and quickly test new ideas like the generalised 
 * weak-constraint and the saddle-point algorithm in the recent past. 
 * Also the complexity and obscurity of the code base constituted a high 
 * initial barrier for new developers and made it difficult to pursue 
 * external collaborations. These considerations led to the idea of 
 * developing a new data assimilation framework based on an object 
 * oriented software paradigm.
 *
 * With the development of OOPS, a more generic top level has been 
 * created that mainly serves the generation of the initial conditions
 * but that can also be considered as a driver that can configure forecasts 
 * with and without coupling even though OOPS will not immediately be run 
 * in operations as the top-level driver infrastructure for forecasts. 
 * It is clear though that OOPS does execute model trajectories for the 
 * comparison between short-range forecasts and observations, and that 
 * this will be performed with a fully coupled system in the future.
 *
 * OOPS has caused a substantial rewrite of the IFS in order for the IFS
 * code to map reasonably to the OOPS object classes and methods. OOPS 
 * represents the model in a non-global view, thus it is cloneable at a 
 * different resolution or with a different functionality in the wider 
 * workflow. 
 *
 * \section DesignSection Design
 *
 * \image html CostFunction.png
 * Fig.1 The principle behind the design of OOPS is to recognise that
 * each element of the cost function is an object. OOPS exposes a set of
 * interface classes, which need to be implemented by models. The interface
 * classes are manipulated at higher level to implement variational 
 * algorithms. 
 *
 * Fig. 1 illustrates a quadratic cost function, where the emphasis is put 
 * on recognizing that it is build from of a set of objects, like the model,
 * increment, observation operator, background and observation error 
 * covariance matrices, etc. The design of OOPS comprises three layers (Fig. 2). 
 * In the middle layer, OOPS defines a set of templated abstract interface classes, 
 * referred to as building blocks. They represent entities manipulated 
 * by variational algorithms like state, increment, control vectors, 
 * covariances, model, observation operators. The building blocks are 
 * combined together by the top layer into applications, like 4D-Var, 
 * forecast, FSOI, etc. Abstract interface classes wrap around 
 * model-specific implementations of those classes through the C++ 
 * templating mechanism. Each abstract interface class receives a model 
 * as a template argument. A model template argument defines a set of 
 * aliases allowing to link generic classes to model-specific 
 * implementations which amount to the bottom layer of the system. The 
 * link only happens at compile time. This allows the separation 
 * of the control top and middle layer from the bottom model-specific layer.
 * Welcome to the documentation for the Object-Oriented Prediction
 * System (OOPS). The key advantage of the OOPS design lies in the anticipated 
 * invariance of the middle layer: the building blocks. It is highly unlikely that 
 * the interfaces will change significantly in the future. The topmost 
 * layer is relatively lightweight and can be quickly redesigned to meet 
 * future requirements.
 * 
 * \image html OOPSdesign.png
 * Fig.2 Conceptual design of OOPS with the application top layer, 
 * the templated building blocks of variational assimilation in the 
 * middle layer and the interfaces to model specific components in 
 * the lowest layer. The main user-exposed features are implemented 
 * via inheritance represented in blue. Ensemble algorithms could be 
 * implemented as yet another application in the future. They would 
 * require, while sharing a subset of interfaces and general building 
 * blocks, a completely new design of the control layer. The 
 * faint-coloured elements (FSOI and SVD) represent additional functionality, 
 * and are not essential for producing the analysis.
 *
 * \subsection TheModelsSubSection The Models
 * Currently, the following models are available:
 * - The Lorenz 1995 (40-variable) model.
 * - A two-layer quasi-geostrophic model.
 *
 * \subsection WhereToStartSubSection Where to Start
 *
 * The following class implements incremental weak-constraint 4D-Var,
 * and may be a suitable starting point for exploring the code:
 * - runs/Variational.h
 *
 * Instruction for installing and running OOPS, as well as information on how
 * to contribute to OOPS development can be found in the README file here:
 *
 * https://git.ecmwf.int/projects/OOPS/repos/oops/browse
 *
 */
#endif  // DOCUMENTS_MAINPAGE_H_
