/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef LORENZ95_INTERPOLATORL95_H_
#define LORENZ95_INTERPOLATORL95_H_

#include <memory> // for std::unique_ptr
#include <ostream>
#include <string>

#include "eckit/config/Configuration.h"
#include "eckit/memory/NonCopyable.h"

#include "util/DateTime.h"
#include "util/ObjectCounter.h"
#include "util/Printable.h"

// Forward declarations
namespace lorenz95 {
  class Resolution;
  class StateL95;
  class IncrementL95;
  class NoVariables;

/// Interpolator matrix for lorenz95 model.

// -----------------------------------------------------------------------------
class InterpolatorL95: public util::Printable,
                            private eckit::NonCopyable,
                            private util::ObjectCounter<InterpolatorL95> {
 public:
    InterpolatorL95(const StateL95 &, const eckit::Configuration &, const bool){}
    InterpolatorL95(const StateL95 &, const Resolution &, const eckit::Configuration &, const bool){}
    explicit InterpolatorL95(const InterpolatorL95 &){}
    ~InterpolatorL95(){}

    InterpolatorL95 & operator=(const InterpolatorL95 &) {return *this;}
    InterpolatorL95 & operator+=(const StateL95 &) {return *this;}
    InterpolatorL95 & operator-=(const StateL95 &) {return *this;}

    void set(const Resolution &,const Resolution &) {}
    virtual void set_vector(const eckit::Configuration &) {}

    void interpolate(const IncrementL95 &, const NoVariables &,  IncrementL95 &) const;
    void interpolate(const StateL95 &, const NoVariables &,  StateL95 &) const;
    void smooth(IncrementL95 &) const{}
    void save(IncrementL95 &) const{}
    void read(const IncrementL95 &){}
    void write(const eckit::Configuration &) const{}

    static const std::string classname() {return "lorenz95::InterpolatorL95";}

 private:
    void print(std::ostream &) const;
  //F90interp keyFtnConfig_;
};
// -----------------------------------------------------------------------------

}  // namespace lorenz95

#endif  // LORENZ95_INTERPOLATORL95_H_
