/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef LORENZ95_L95TRAITS_H_
#define LORENZ95_L95TRAITS_H_

#include <string>

#include "lorenz95/ErrorCovarianceL95.h"
#include "lorenz95/GomL95.h"
#include "lorenz95/IncrementL95.h"
#include "lorenz95/IncrEnsCtlVecL95.h"
#include "lorenz95/IncrModCtlVecL95.h"
#include "lorenz95/LocsL95.h"
#include "lorenz95/LocalizationMatrixL95.h"
#include "lorenz95/ModelBias.h"
#include "lorenz95/ModelBiasCtlVec.h"
#include "lorenz95/ModelBiasCorrection.h"
#include "lorenz95/ModelBiasCovariance.h"
#include "lorenz95/ModelBiasEstimator.h"
#include "lorenz95/ModelL95.h"
#include "lorenz95/NoVariables.h"
#include "lorenz95/ObsTable.h"
#include "lorenz95/ObsVec1D.h"
#include "lorenz95/Resolution.h"
#include "lorenz95/StateL95.h"
#include "lorenz95/InterpolatorL95.h"
#include "lorenz95/HorizScaleDecompositionL95.h"

namespace lorenz95 {

struct L95Traits {
  static std::string name() {return "Lorenz 95";}

  using Geometry = lorenz95::Resolution;
  using Variables = lorenz95::NoVariables;

  using State = lorenz95::StateL95;
  using Model = lorenz95::ModelL95;
  using Increment = lorenz95::IncrementL95;
  using IncrEnsCtlVec = lorenz95::IncrEnsCtlVecL95;
  using IncrModCtlVec = lorenz95::IncrModCtlVecL95;
  using Covariance = lorenz95::ErrorCovarianceL95;

  using ModelAuxControl = lorenz95::ModelBias;
  using ModelAuxControlEstimator = lorenz95::ModelBiasEstimator;
  using ModelAuxCtlVec = lorenz95::ModelBiasCtlVec;
  using ModelAuxIncrement = lorenz95::ModelBiasCorrection;
  using ModelAuxCovariance = lorenz95::ModelBiasCovariance;

  using ObsSpace = lorenz95::ObsTable;
  using ObsVector = lorenz95::ObsVec1D;

  using GeoVaLs = lorenz95::GomL95;
  using Locations = lorenz95::LocsL95;

  using LocalizationMatrix = lorenz95::LocalizationMatrixL95;
  using Interpolator = lorenz95::InterpolatorL95;
  using HorizScaleDecomposition = lorenz95::HorizScaleDecompositionL95;
};

}  // namespace lorenz95

#endif  // LORENZ95_L95TRAITS_H_
