/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "lorenz95/ObsBias.h"

#include <iostream>
#include <string>

#include "eckit/config/Configuration.h"

#include "util/Logger.h"

#include "lorenz95/ObsBiasCorrection.h"

// -----------------------------------------------------------------------------
namespace lorenz95 {
// -----------------------------------------------------------------------------
static oops::ObsAuxControlMaker<L95Traits, ObsBias> makesObsBias_("ObsBiasL95");
// -----------------------------------------------------------------------------

ObsBias::ObsBias(const ObsTable &, const eckit::Configuration & conf)
  : bias_(0.0), active_(false), obsSpaceKey_(0), dummyKey_(0)
{
  oops::Log::trace() << "ObsBias::ObsBias conf is:" << conf << std::endl;
  if (conf.has("bias")) {
    bias_ = conf.getDouble("bias");
    active_ = true;
    oops::Log::info() << "ObsBias::ObsBias created, bias = " << bias_ << std::endl;
  }
}
// -----------------------------------------------------------------------------
ObsBias::ObsBias(const ObsBias & other, const bool copy)
  : bias_(0.0), active_(other.active_), obsSpaceKey_(other.obsSpaceKey_),
    dummyKey_(other.dummyKey_)
{
  if (active_ && copy) bias_ = other.bias_;
}
// -----------------------------------------------------------------------------
ObsBias* ObsBias::clone(const bool copy) const {
  return new ObsBias(*this, copy);
}
// -----------------------------------------------------------------------------
oops::ObsAuxControlBase<L95Traits> & ObsBias::operator+=(const oops::ObsAuxIncrementBase<L95Traits> & dx) {
  if (active_) {
    const ObsBiasCorrection * pdx = dynamic_cast<const ObsBiasCorrection*>(&dx);
    bias_ += (*pdx).value();
  }
  return *this;
}
// -----------------------------------------------------------------------------
void ObsBias::print(std::ostream & os) const {
  if (active_) {os << std::endl << "ObsBias = " << bias_;}
}
// -----------------------------------------------------------------------------
}  // namespace lorenz95
