/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef LORENZ95_OBSBIASCTLVEC_H_
#define LORENZ95_OBSBIASCTLVEC_H_

#include <iostream>
#include <vector>

#include "oops/interface/ObsAuxCtlVecBase.h"
#include "util/Printable.h"

#include "lorenz95/L95Traits.h"

namespace oops {
  template <typename MODEL>
  class ObsAuxCovarianceBase;
}

namespace eckit {
  class Configuration;
}

namespace lorenz95 {

  /// ObsBiasCtlVec Class
  /*!
   *  Part of the control vector corresponding to the ObsAuxIncrement part of the
   *  control increment
   */

// -----------------------------------------------------------------------------

class ObsBiasCtlVec : public oops::ObsAuxCtlVecBase<L95Traits> {
 public:
/// Constructor, destructor
  ObsBiasCtlVec(const oops::ObsAuxCovarianceBase<L95Traits> &) {}
  ObsBiasCtlVec(const oops::ObsAuxCovarianceBase<L95Traits> &, const ObsBiasCtlVec &) {}
  ObsBiasCtlVec(const ObsBiasCtlVec &, const bool copy = true) {}
  virtual ~ObsBiasCtlVec() {}

/// Cloning
  virtual ObsBiasCtlVec* clone(const oops::ObsAuxCovarianceBase<L95Traits> &) const;
  virtual ObsBiasCtlVec* clone(const bool copy = true) const;

/// Linear algebra operators
  void zero() {}
  ObsBiasCtlVec & operator=(const oops::ObsAuxCtlVecBase<L95Traits> &) {return *this;}
  ObsBiasCtlVec & operator+=(const oops::ObsAuxCtlVecBase<L95Traits> &) {return *this;}
  ObsBiasCtlVec & operator-=(const oops::ObsAuxCtlVecBase<L95Traits> &) {return *this;}
  ObsBiasCtlVec & operator*=(const double &) {return *this;}
  void axpy(const double &, const oops::ObsAuxCtlVecBase<L95Traits> &) {}
  double dot_product_with(const oops::ObsAuxCtlVecBase<L95Traits> &) const {return 0.0;}

/// I/O and diagnostics
  void read(const eckit::Configuration &) {}
  void write(const eckit::Configuration &) const {}
  double norm() const {return 0.0;}

/// Interfacing
  const int & getAccess() const {return dummyKey_;}
  int & getAccess() {return dummyKey_;}

 private:
  void print(std::ostream &) const {}
  int dummyKey_;
};

// -----------------------------------------------------------------------------

}  // namespace lorenz95

#endif  // LORENZ95_OBSBIASCTLVEC_H_
