/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_GEOMETRYQG_H_
#define QG_MODEL_GEOMETRYQG_H_

#include <ostream>
#include <string>

#include "atlas/field.h"
#include "atlas/functionspace.h"

#include "util/ObjectCounter.h"
#include "util/Printable.h"

#include "model/QgFortran.h"
#include "model/VariablesQG.h"

namespace eckit {
  class Configuration;
}

namespace qg {

// -----------------------------------------------------------------------------
/// GeometryQG handles geometry for QG model.

class GeometryQG : public util::Printable,
                   private util::ObjectCounter<GeometryQG> {
 public:
  static const std::string classname() {return "qg::GeometryQG";}

  explicit GeometryQG(const eckit::Configuration &);
  GeometryQG(const GeometryQG &);
  ~GeometryQG();

  /// Returns number of levels for each variable
  std::vector<size_t> variableSizes(const VariablesQG &) const;

  /// Accessor to the functionspace
  const atlas::FunctionSpace & functionSpace() const {return *functionSpace_;}

  /// Accessor to the geometry fields
  const atlas::FieldSet & fields() const {return *fields_;}

  /// Levels direction (no importance here)
  bool levelsAreTopDown() const {return false;}

  /// Communicators
  const eckit::mpi::Comm & getComm() const
    {return eckit::mpi::comm();}
  const eckit::mpi::Comm & timeComm() const
    {return eckit::mpi::self();}

  F90geom & toFortran() {return keyGeom_;}
  const F90geom & toFortran() const {return keyGeom_;}

 private:
  GeometryQG & operator=(const GeometryQG &);
  void print(std::ostream &) const;
  F90geom keyGeom_;
  std::unique_ptr<atlas::functionspace::StructuredColumns> functionSpace_;
  std::unique_ptr<atlas::FieldSet> fields_;
};
// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_GEOMETRYQG_H_
