/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_INTERPOLATORQG_H_
#define QG_MODEL_INTERPOLATORQG_H_

#include <memory> // for std::unique_ptr
#include <ostream>
#include <string>

#include "eckit/config/Configuration.h"
#include "eckit/memory/NonCopyable.h"

#include "util/DateTime.h"
#include "util/ObjectCounter.h"
#include "util/Printable.h"

#include "model/QgFortran.h"

// Forward declarations
namespace qg {
  class GeometryQG;
  class StateQG;
  class IncrementQG;
  class VariablesQG;

/// Interpolator matrix for QG model.

// -----------------------------------------------------------------------------
class InterpolatorQG: public util::Printable,
                            private eckit::NonCopyable,
                            private util::ObjectCounter<InterpolatorQG> {
 public:
    InterpolatorQG(const StateQG &, const eckit::Configuration &, const bool){}
    InterpolatorQG(const StateQG &, const GeometryQG &, const eckit::Configuration &, const bool){}
    explicit InterpolatorQG(const InterpolatorQG &){}
    ~InterpolatorQG(){}

    InterpolatorQG & operator=(const InterpolatorQG &) {return *this;}
    InterpolatorQG & operator+=(const StateQG &) {return *this;}
    InterpolatorQG & operator-=(const StateQG &) {return *this;}

    void set(const GeometryQG &,const GeometryQG &) {}
    virtual void set_vector(const eckit::Configuration &) {}

    void interpolate(const IncrementQG &, const VariablesQG &,  IncrementQG &) const;
    void interpolate(const StateQG &, const VariablesQG &,  StateQG &) const;
    void smooth(IncrementQG &) const{}
    void save(IncrementQG &) const{}
    void read(const IncrementQG &){}
    void write(const eckit::Configuration &) const{}

    static const std::string classname() {return "qg::InterpolatorQG";}

 private:
    void print(std::ostream &) const;
  //F90interp keyFtnConfig_;
};
// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_INTERPOLATORQG_H_
