/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_MODELBIASINCREMENT_H_
#define QG_MODEL_MODELBIASINCREMENT_H_

#include <iostream>
#include <memory> // for std::unique_ptr

#include "util/Printable.h"

#include "model/FieldsQG.h"
#include "model/GeometryQG.h"

namespace eckit {
  class Configuration;
}

namespace qg {
  class ModelBias;
  class ModelBiasCovariance;
  class GeometryQG;

// -----------------------------------------------------------------------------

class ModelBiasIncrement : public util::Printable {
 public:
/// Constructor, destructor
  ModelBiasIncrement(const GeometryQG &, const eckit::Configuration &);
  ModelBiasIncrement(const ModelBiasIncrement &, const bool = true);
  ModelBiasIncrement(const ModelBiasIncrement &, const eckit::Configuration &);
  ~ModelBiasIncrement();

/// Linear algebra operators
  void diff(const ModelBias &, const ModelBias &);
  void zero();
  ModelBiasIncrement & operator=(const ModelBiasIncrement &);
  ModelBiasIncrement & operator+=(const ModelBiasIncrement &);
  ModelBiasIncrement & operator-=(const ModelBiasIncrement &);
  ModelBiasIncrement & operator*=(const double &);
  void axpy(const double, const ModelBiasIncrement &);
  double dot_product_with(const ModelBiasIncrement &) const;

/// I/O and diagnostics
  void read(const eckit::Configuration &);
  void write(const eckit::Configuration &) const;
  double norm() const;

/// Access to fields
  FieldsQG & fields() {return *fields_;}
  const FieldsQG & fields() const {return *fields_;}

/// Status
  const bool & isActive() const {return active_;}

 private:
  void print(std::ostream & os) const;
  bool active_;
  std::unique_ptr<FieldsQG> fields_;
  std::unique_ptr<FieldsQG> stash_;
};

// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_MODELBIASINCREMENT_H_
