/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_OBSERR_STREAMQG_H
#define QG_OBSERR_STREAMQG_H

#include <memory> // for std::unique_ptr
#include <sstream>
#include <string>

#include "oops/interface/ObsAuxControlBase.h"
#include "oops/interface/ObsErrorBase.h"

#include "QgTraits.h"

namespace eckit {
  class Configuration;
}

namespace qg {

// -----------------------------------------------------------------------------
/// Diagonal observation error covariance matrix.

class ObsErrStreamQG : public oops::ObsErrorBase<QgTraits> {
  using ObsAuxControlPtrMap_ =
    typename std::map<std::string,
                      std::unique_ptr<oops::ObsAuxControlBase<QgTraits>>>;

 public:
  ObsErrStreamQG(const ObsSpaceQG &, const eckit::Configuration &);
  ~ObsErrStreamQG();

/// Linearize and reset for inner loop (nothing in this simple case)
  void linearize(const ObsVecQG &);

/// Linearize and reset for inner loop (nothing in this simple case)
  void linearize(const ObsAuxControlPtrMap_ &,
                 const ObsVecQG &dy);

/// Setup VarQC weights
  void setupWeights(const ObsVecQG &);

/// Multiply a Departure by \f$R\f$
  ObsVecQG * multiply(const ObsVecQG &) const;

/// Multiply a Departure by \f$R^{-1}\f$
  ObsVecQG * inverseMultiply(const ObsVecQG &) const;
  
/// Multiply a Departure by \f$R^{-1/2}\f$
  ObsVecQG * inverseMultiplySqrt(const ObsVecQG &) const;
  
/// Multiply a Departure by \f$R^{-T/2}\f$
  ObsVecQG * inverseMultiplySqrtTrans(const ObsVecQG &) const;

/// Multiply a Departure by \f$W^1/2\f$
  ObsVecQG * multiplyWghtSqrt(const ObsVecQG &) const;

/// Multiply a Departure by \f$W\f$
  ObsVecQG * multiplyWght(const ObsVecQG &) const;

/// Generate random perturbation
  void randomize(ObsVecQG &) const;

/// Get mean error for Jo table
  double getRMSE() const {return stddev_->rms();}

 private:
  void print(std::ostream &) const;

  mutable bool lvarqc_;
  mutable double cleft_;
  mutable double cright_;

  const ObsSpaceQG & obsgeom_;

  std::unique_ptr<ObsVecQG> stddev_;
  std::unique_ptr<ObsVecQG> inverseVariance_;
  std::unique_ptr<ObsVecQG> inverseVarianceSqrt_;
  std::unique_ptr<ObsVecQG> wghtsqrt_;
  std::unique_ptr<ObsVecQG> wght_;
};

// =============================================================================

}  // namespace qg

#endif  // QG_OBSERR_STREAMQG_H
