/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_OBSWSPEEDTLAD_H_
#define QG_MODEL_OBSWSPEEDTLAD_H_

#include <map>
#include <memory>
#include <string>

#include "oops/interface/LinearObsOperBase.h"
#include "util/ObjectCounter.h"
#include "model/QgTraits.h"

// Forward declarations
namespace eckit {
  class Configuration;
}

namespace qg {
  class GomQG;
  class ObsBias;
  class ObsBiasIncrement;
  class ObsSpaceQG;
  class ObsVecQG;

// -----------------------------------------------------------------------------

class ObsWSpeedTLAD : public oops::LinearObsOperBase<QgTraits>,
                      private util::ObjectCounter<ObsWSpeedTLAD> {
  using ObsAuxControlPtrMap_ =
    typename std::map<std::string,
      std::unique_ptr<oops::ObsAuxControlBase<QgTraits>> >;
  using ObsAuxIncrementPtrMap_ =
    typename std::map<std::string,
      std::unique_ptr<oops::ObsAuxIncrementBase<QgTraits>> >;
 public:
  static const std::string classname() {return "qg::ObsWSpeedTLAD";}

  ObsWSpeedTLAD(const ObsSpaceQG &, const eckit::Configuration &);
  virtual ~ObsWSpeedTLAD();

// Obs Operators
  void setTrajectory(const GomQG &,
                     const ObsAuxControlPtrMap_ &);
  void obsEquivTL(const GomQG &, ObsVecQG &,
                  const ObsAuxIncrementPtrMap_ &) const;
  void obsEquivAD(GomQG &, const ObsVecQG &,
                  ObsAuxIncrementPtrMap_ &) const;
  void obsBiasEquivTL(const GomQG &, ObsVecQG &,
                      const ObsAuxIncrementPtrMap_ &) const {}
  void obsBiasEquivAD(const GomQG &, const ObsVecQG &,
                      ObsAuxIncrementPtrMap_ &) const {}

// Other
  std::shared_ptr<const VariablesQG> variables() const override {return varin_;}

  int & toFortran() {return keyOperWspeed_;}
  const int & toFortran() const {return keyOperWspeed_;}

 private:
  void print(std::ostream &) const override;
  F90hop keyOperWspeed_;
  GomQG traj_;
  std::shared_ptr<const VariablesQG> varin_;
};
// -----------------------------------------------------------------------------

}  // namespace qg
#endif  // QG_MODEL_OBSWSPEEDTLAD_H_
