/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_OBSWINDQG_H_
#define QG_MODEL_OBSWINDQG_H_

#include <map>
#include <memory>
#include <ostream>
#include <string>

#include "oops/interface/ObsOperatorBase.h"
#include "util/ObjectCounter.h"

#include "model/ObsSpaceQG.h"
#include "model/QgTraits.h"

// Forward declarations
namespace eckit {
  class Configuration;
}

namespace qg {
  class GomQG;
  class LocationsQG;
  class ObsBias;
  class ObsBiasIncrement;
  class ObsVecQG;

// -----------------------------------------------------------------------------
/// Wind observation for QG model.

class ObsWindQG : public oops::ObsOperatorBase<QgTraits>,
                  private util::ObjectCounter<ObsWindQG> {
  using ObsAuxControlPtrMap_ =
    typename std::map<std::string,
      std::unique_ptr<oops::ObsAuxControlBase<QgTraits>> >;

 public:
  static const std::string classname() {return "qg::ObsWindQG";}

  ObsWindQG(const ObsSpaceQG &, const eckit::Configuration &);
  virtual ~ObsWindQG();

// Obs Operators
  void obsEquiv(const GomQG &, ObsVecQG &,
                const ObsAuxControlPtrMap_ &) const;

  void obsBiasEquiv(const GomQG &, ObsVecQG &,
                    const ObsAuxControlPtrMap_ &) const;

// Other
  std::shared_ptr<const VariablesQG> variables() const {return varin_;}

  int & toFortran() {return keyOperWind_;}
  const int & toFortran() const {return keyOperWind_;}

 private:
  void print(std::ostream &) const;
  F90hop keyOperWind_;
  std::shared_ptr<const VariablesQG> varin_;
};
// -----------------------------------------------------------------------------

}  // namespace qg
#endif  // QG_MODEL_OBSWINDQG_H_
