/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_QGTRAITS_H_
#define QG_MODEL_QGTRAITS_H_

#include <string>

#include "model/ErrorCovarianceQG.h"
#include "model/GeometryQG.h"
#include "model/GomQG.h"
#include "model/IncrementQG.h"
#include "model/IncrEnsCtlVecQG.h"
#include "model/IncrModCtlVecQG.h"
#include "model/LocalizationMatrixQG.h"
#include "model/LocationsQG.h"
#include "model/ModelQG.h"
#include "model/ModelBias.h"
#include "model/ModelBiasCovariance.h"
#include "model/ModelBiasCtlVec.h"
#include "model/ModelBiasEstimator.h"
#include "model/ModelBiasIncrement.h"
#include "model/ObsSpaceQG.h"
#include "model/ObsVecQG.h"
#include "model/StateQG.h"
#include "model/VariablesQG.h"
#include "model/InterpolatorQG.h"
#include "model/HorizScaleDecompositionQG.h"

namespace qg {

struct QgTraits {
  static std::string name() {return "QG";}

  using Geometry = qg::GeometryQG;
  using Variables = qg::VariablesQG;

  using State = qg::StateQG;
  using Model = qg::ModelQG;
  using Increment = qg::IncrementQG;
  using IncrEnsCtlVec = qg::IncrEnsCtlVecQG;
  using IncrModCtlVec = qg::IncrModCtlVecQG;
  using Covariance = qg::ErrorCovarianceQG;

  using ModelAuxControl = qg::ModelBias;
  using ModelAuxControlEstimator = qg::ModelBiasEstimator;
  using ModelAuxIncrement = qg::ModelBiasIncrement;
  using ModelAuxCtlVec = qg::ModelBiasCtlVec;
  using ModelAuxCovariance = qg::ModelBiasCovariance;

  using ObsSpace = qg::ObsSpaceQG;
  using ObsVector = qg::ObsVecQG;

  using GeoVaLs = qg::GomQG;
  using Locations = qg::LocationsQG;

  using LocalizationMatrix = qg::LocalizationMatrixQG;
  using Interpolator = qg::InterpolatorQG;
  using HorizScaleDecomposition = qg::HorizScaleDecompositionQG;
};

}  // namespace qg

#endif  // QG_MODEL_QGTRAITS_H_
