! (C) Copyright 2009-2016 ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
! In applying this licence, ECMWF does not waive the privileges and immunities 
! granted to it by virtue of its status as an intergovernmental organisation nor
! does it submit to any jurisdiction.

!> Structure holding configuration variables for the QG model

module qg_configs

use iso_c_binding
use kinds
use pm_linked_list_mod

implicit none
private
public :: qg_config
public :: config_list, get_config

!> Fortran derived type to hold configuration data for the QG model
type :: qg_config
  integer :: nx     !< Zonal grid dimension
  integer :: ny     !< Meridional grid dimension
  ! dimensional parameters
  real(kind=kind_real) :: dt0       !< dimensional time (seconds)
  real(kind=kind_real) :: d1        !< depth of top layer (m)
  real(kind=kind_real) :: d2        !< depth of bottom layer (m)
  real(kind=kind_real) :: deltax0   !< zonal grid spacing (m)
  real(kind=kind_real) :: deltay0   !< meridional grid spacing (m)
  ! non-dimensional parameters
  real(kind=kind_real) :: dt        !< non-dimensional time
  real(kind=kind_real) :: f1        !< Coefficient of PV operator
  real(kind=kind_real) :: f2        !< Coefficient of PV operator
  real(kind=kind_real) :: deltax    !< non-dimensional zonal grid spacing
  real(kind=kind_real) :: deltay    !< non-dimensional meridional grid spacing
  real(kind=kind_real) :: rsmax     !< max non-dimensional height of orography
  real(kind=kind_real), allocatable :: rs(:,:) !< non-dimensional orography
end type qg_config

!> Global list
type(pm_linked_list) :: config_list

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

function get_config(c_self)
integer(kind=c_int), intent(in) :: c_self
type(qg_config), pointer :: get_config
class(*), pointer :: this=>null()

call config_list%get(c_self, this)

select type(self => this)
type is (qg_config)
  get_config=>self
class default
  call abor1_ftn('get_config : unexpected type ')
end select

end function get_config

! ------------------------------------------------------------------------------
end module qg_configs
