! (C) Copyright 2009-2016 ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
! In applying this licence, ECMWF does not waive the privileges and immunities 
! granted to it by virtue of its status as an intergovernmental organisation nor
! does it submit to any jurisdiction.

!> Interfaces to be called from C++ for Fortran handling of QG model fields

! ------------------------------------------------------------------------------

subroutine qg_field_create_c(c_key_self, c_key_geom, c_key_vars) bind(c,name='qg_field_create_f90')
use iso_c_binding
use qg_fields
use qg_geom_mod
use qg_vars_mod
implicit none
integer(c_int), intent(inout) :: c_key_self
integer(c_int), intent(in) :: c_key_geom !< Geometry
integer(c_int), intent(in) :: c_key_vars !< List of variables

type(qg_field)          :: template
type(qg_field), pointer :: self
type(qg_geom),  pointer :: geom
type(qg_vars),  pointer :: vars

! -- get objects

call fields_list%add(c_key_self,template)

self => get_fields(c_key_self)
geom => get_geom(c_key_geom)
vars => get_vars(c_key_vars)

call create(self, geom, vars)

end subroutine qg_field_create_c

! ------------------------------------------------------------------------------

subroutine qg_field_delete_c(c_key_self) bind(c,name='qg_field_delete_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(inout) :: c_key_self
type(qg_field), pointer :: self

! -- get objects

self => get_fields(c_key_self)

call delete(self)

call fields_list%remove(c_key_self)

end subroutine qg_field_delete_c

! ------------------------------------------------------------------------------

subroutine qg_field_zero_c(c_key_self) bind(c,name='qg_field_zero_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in) :: c_key_self
type(qg_field), pointer :: self

! -- get objects

self => get_fields(c_key_self)

call zeros(self)

end subroutine qg_field_zero_c

! ------------------------------------------------------------------------------

subroutine qg_field_random_c(c_key_self) bind(c,name='qg_field_random_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in) :: c_key_self
type(qg_field), pointer :: self

! -- get objects

self => get_fields(c_key_self)

call random(self)

end subroutine qg_field_random_c

! ------------------------------------------------------------------------------

subroutine qg_field_copy_c(c_key_self,c_key_rhs) bind(c,name='qg_field_copy_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in) :: c_key_self
integer(c_int), intent(in) :: c_key_rhs

type(qg_field), pointer :: self
type(qg_field), pointer :: rhs

! -- get objects

self => get_fields(c_key_self)
rhs => get_fields(c_key_rhs)

call copy(self, rhs)

end subroutine qg_field_copy_c

! ------------------------------------------------------------------------------

subroutine qg_field_self_add_c(c_key_self,c_key_rhs) bind(c,name='qg_field_self_add_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in) :: c_key_self
integer(c_int), intent(in) :: c_key_rhs

type(qg_field), pointer :: self
type(qg_field), pointer :: rhs

self => get_fields(c_key_self)
rhs => get_fields(c_key_rhs)

call self_add(self,rhs)

end subroutine qg_field_self_add_c

! ------------------------------------------------------------------------------

subroutine qg_field_self_schur_c(c_key_self,c_key_rhs) bind(c,name='qg_field_self_schur_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in) :: c_key_self
integer(c_int), intent(in) :: c_key_rhs

type(qg_field), pointer :: self
type(qg_field), pointer :: rhs

! -- get objects

self => get_fields(c_key_self)
rhs => get_fields(c_key_rhs)

call self_schur(self,rhs)

end subroutine qg_field_self_schur_c

! ------------------------------------------------------------------------------

subroutine qg_field_self_sub_c(c_key_self,c_key_rhs) bind(c,name='qg_field_self_sub_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in) :: c_key_self
integer(c_int), intent(in) :: c_key_rhs

type(qg_field), pointer :: self
type(qg_field), pointer :: rhs

! -- get objects

self => get_fields(c_key_self)
rhs => get_fields(c_key_rhs)

call self_sub(self,rhs)

end subroutine qg_field_self_sub_c

! ------------------------------------------------------------------------------

subroutine qg_field_self_mul_c(c_key_self,c_zz) bind(c,name='qg_field_self_mul_f90')
use iso_c_binding
use qg_fields
use kinds
implicit none
integer(c_int), intent(in) :: c_key_self
real(c_double), intent(in) :: c_zz
type(qg_field), pointer :: self
real(kind=kind_real) :: zz

! -- get objects

self => get_fields(c_key_self)
zz = c_zz

call self_mul(self,zz)

end subroutine qg_field_self_mul_c

! ------------------------------------------------------------------------------

subroutine qg_field_axpy_c(c_key_self,c_zz,c_key_rhs) bind(c,name='qg_field_axpy_f90')
use iso_c_binding
use qg_fields
use kinds
implicit none
integer(c_int), intent(in) :: c_key_self
real(c_double), intent(in) :: c_zz
integer(c_int), intent(in) :: c_key_rhs

type(qg_field), pointer :: self
type(qg_field), pointer :: rhs
real(kind=kind_real) :: zz

! -- get objects

self => get_fields(c_key_self)
rhs => get_fields(c_key_rhs)
zz = c_zz

call axpy(self,zz,rhs)

end subroutine qg_field_axpy_c

! ------------------------------------------------------------------------------

subroutine qg_field_dot_prod_c(c_key_fld1,c_key_fld2,c_prod) bind(c,name='qg_field_dot_prod_f90')
use iso_c_binding
use qg_fields
use kinds
implicit none
integer(c_int), intent(in)    :: c_key_fld1, c_key_fld2
real(c_double), intent(inout) :: c_prod
real(kind=kind_real) :: zz
type(qg_field), pointer :: fld1, fld2

! -- get objects

fld1 => get_fields(c_key_fld1)
fld2 => get_fields(c_key_fld2)

call dot_prod(fld1,fld2,zz)

c_prod = zz

end subroutine qg_field_dot_prod_c

! ------------------------------------------------------------------------------

subroutine qg_field_dirac_c(c_key_fld, c_conf) bind(c,name='qg_field_dirac_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in)    :: c_key_fld
type(c_ptr), intent(in)    :: c_conf

type(qg_field), pointer :: fld

! -- get objects

fld => get_fields(c_key_fld)

call dirac(fld, c_conf)

end subroutine qg_field_dirac_c

! ------------------------------------------------------------------------------

subroutine qg_field_add_incr_c(c_key_self,c_key_rhs) bind(c,name='qg_field_add_incr_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in) :: c_key_self
integer(c_int), intent(in) :: c_key_rhs
type(qg_field), pointer :: self
type(qg_field), pointer :: rhs

! -- get objects

self => get_fields(c_key_self)
rhs => get_fields(c_key_rhs)

call add_incr(self,rhs)

end subroutine qg_field_add_incr_c

! ------------------------------------------------------------------------------

subroutine qg_field_diff_incr_c(c_key_lhs,c_key_x1,c_key_x2) bind(c,name='qg_field_diff_incr_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in) :: c_key_lhs
integer(c_int), intent(in) :: c_key_x1
integer(c_int), intent(in) :: c_key_x2
type(qg_field), pointer :: lhs
type(qg_field), pointer :: x1
type(qg_field), pointer :: x2

! -- get objects

lhs => get_fields(c_key_lhs)
x1 => get_fields(c_key_x1)
x2 => get_fields(c_key_x2)

call diff_incr(lhs,x1,x2)

end subroutine qg_field_diff_incr_c

! ------------------------------------------------------------------------------

subroutine qg_field_change_resol_c(c_key_fld,c_key_rhs) bind(c,name='qg_field_change_resol_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in) :: c_key_fld
integer(c_int), intent(in) :: c_key_rhs
type(qg_field), pointer :: fld, rhs

! -- get objects

fld => get_fields(c_key_fld)
rhs => get_fields(c_key_rhs)

call change_resol(fld,rhs)

end subroutine qg_field_change_resol_c

! ------------------------------------------------------------------------------

subroutine qg_field_read_file_c(c_key_fld, c_conf, c_dt) bind(c,name='qg_field_read_file_f90')
use iso_c_binding
use qg_fields
use datetime_mod

implicit none
integer(c_int), intent(in) :: c_key_fld  !< Fields
type(c_ptr), intent(in)    :: c_conf !< Configuration
type(c_ptr), intent(inout) :: c_dt   !< DateTime

type(qg_field), pointer :: fld
type(datetime) :: fdate

! -- get objects

fld => get_fields(c_key_fld)

call c_f_datetime(c_dt, fdate)
call read_file(fld, c_conf, fdate)

end subroutine qg_field_read_file_c

! ------------------------------------------------------------------------------

subroutine qg_field_write_file_c(c_key_fld, c_conf, c_dt) bind(c,name='qg_field_write_file_f90')
use iso_c_binding
use qg_fields
use datetime_mod

implicit none
integer(c_int), intent(in) :: c_key_fld  !< Fields
type(c_ptr), intent(in) :: c_conf !< Configuration
type(c_ptr), intent(in) :: c_dt   !< DateTime

type(qg_field), pointer :: fld
type(datetime) :: fdate

! -- get objects

fld => get_fields(c_key_fld)

call c_f_datetime(c_dt, fdate)
call write_file(fld, c_conf, fdate)

end subroutine qg_field_write_file_c

! ------------------------------------------------------------------------------

subroutine qg_field_gpnorm_c(c_key_fld, kf, pstat) bind(c,name='qg_field_gpnorm_f90')
use iso_c_binding
use qg_fields
use kinds
implicit none
integer(c_int), intent(in) :: c_key_fld
integer(c_int), intent(in) :: kf
real(c_double), intent(inout) :: pstat(3*kf)

type(qg_field), pointer :: fld
real(kind=kind_real) :: zstat(3, kf)
integer :: jj, js, jf

! -- get objects

fld => get_fields(c_key_fld)

call gpnorm(fld, kf, zstat)
jj=0
do jf = 1, kf
  do js = 1, 3
    jj=jj+1
    pstat(jj) = zstat(js,jf)
  enddo
enddo

end subroutine qg_field_gpnorm_c

! ------------------------------------------------------------------------------

subroutine qg_field_rms_c(c_key_fld, prms) bind(c,name='qg_field_rms_f90')
use iso_c_binding
use qg_fields
use kinds
implicit none
integer(c_int), intent(in) :: c_key_fld
real(c_double), intent(inout) :: prms

type(qg_field), pointer :: fld
real(kind=kind_real) :: zz

! -- get objects

fld => get_fields(c_key_fld)

call fldrms(fld, zz)

prms = zz

end subroutine qg_field_rms_c

! ------------------------------------------------------------------------------

subroutine qg_field_interp_tl_c(c_key_fld,c_key_loc,c_key_gom) bind(c,name='qg_field_interp_tl_f90')
use iso_c_binding
use qg_fields
use qg_locs_mod
use qg_goms_mod
implicit none
integer(c_int), intent(in) :: c_key_fld
integer(c_int), intent(in) :: c_key_loc
integer(c_int), intent(in) :: c_key_gom
type(qg_field), pointer :: fld
type(qg_locs),  pointer :: locs
type(qg_goms),  pointer :: gom

! -- get objects

fld => get_fields(c_key_fld)
locs => get_locs(c_key_loc)
gom => get_goms(c_key_gom)

call interp_tl(fld, locs, gom)

end subroutine qg_field_interp_tl_c

! ------------------------------------------------------------------------------

subroutine qg_field_interp_ad_c(c_key_fld,c_key_loc,c_key_gom) bind(c,name='qg_field_interp_ad_f90')
use iso_c_binding
use qg_fields
use qg_locs_mod
use qg_goms_mod
implicit none
integer(c_int), intent(in) :: c_key_fld
integer(c_int), intent(in) :: c_key_loc
integer(c_int), intent(in) :: c_key_gom
type(qg_field), pointer :: fld
type(qg_locs),  pointer :: locs
type(qg_goms),  pointer :: gom

! -- get objects

fld => get_fields(c_key_fld)
locs => get_locs(c_key_loc)
gom => get_goms(c_key_gom)

call interp_ad(fld, locs, gom)

end subroutine qg_field_interp_ad_c

! ------------------------------------------------------------------------------

subroutine qg_fieldnum_c(c_key_fld, nx, ny, nf, nb) bind(c,name='qg_field_sizes_f90')
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in) :: c_key_fld
integer(kind=c_int), intent(inout) :: nx, ny, nf, nb
type(qg_field), pointer :: fld

! -- get objects

fld => get_fields(c_key_fld)

nx = fld%nx
ny = fld%ny
nf = fld%nf
nb =0
if (fld%lbc) nb = 2

end subroutine qg_fieldnum_c

! ------------------------------------------------------------------------------

subroutine qg_field_serialize_c(c_archive, c_key_fld) bind(c,name='qg_field_serialize_f90')
  use kinds
  use iso_c_binding
  use qg_fields

  implicit none

  type(c_ptr)        , intent(inout) :: c_archive
  integer(kind=c_int), intent(in)    :: c_key_fld

  type(qg_field), pointer :: fld
  type(c_ptr)             :: cptr
  integer                 :: isize

  interface
  subroutine serialize(c_serializer, nb_data, data ) bind(C,name='ftn_serialize')
    use iso_c_binding
    implicit none
    type(c_ptr), value :: c_serializer
    integer(kind=c_int), value :: nb_data
    type(c_ptr), value :: data
  end subroutine serialize
  end interface

  fld => get_fields(c_key_fld)
  cptr = c_loc(fld%gfld3d(1,1,1))
  isize = size(fld%gfld3d)*kind_real
  call serialize(c_archive, isize, cptr)

end subroutine qg_field_serialize_c

! ------------------------------------------------------------------------------

subroutine qg_field_deserialize_c(c_archive, c_key_fld) bind(c,name='qg_field_deserialize_f90')
  use kinds
  use iso_c_binding
  use qg_fields

  implicit none

  type(c_ptr)        , intent(inout) :: c_archive
  integer(kind=c_int), intent(in)    :: c_key_fld

  type(qg_field), pointer :: fld
  type(c_ptr)             :: cptr
  integer                 :: isize

  interface
  subroutine deserialize(c_serializer, nb_data, data ) bind(C,name='ftn_serialize')
    use iso_c_binding
    implicit none
    type(c_ptr), value :: c_serializer
    integer(kind=c_int), value :: nb_data
    type(c_ptr), value :: data
  end subroutine deserialize
  end interface

  fld => get_fields(c_key_fld)
  cptr = c_loc(fld%gfld3d(1,1,1))
  isize = size(fld%gfld3d)*kind_real
  call deserialize(c_archive, isize, cptr)

end subroutine qg_field_deserialize_c

! ------------------------------------------------------------------------------

subroutine qg_field_to_fieldset_c(c_key_fld, c_fspace, c_fset) bind(c,name='qg_field_to_fieldset_f90')
use atlas_module
use iso_c_binding
use qg_fields
use qg_geom_mod
implicit none
integer(c_int), intent(in)     :: c_key_fld
type(c_ptr), intent(in), value :: c_fspace
type(c_ptr), intent(in), value :: c_fset

type(qg_field), pointer :: fld
type(atlas_functionspace_pointcloud) :: fspace
type(atlas_fieldset) :: fset

! -- get objects

fld => get_fields(c_key_fld)
fspace = atlas_functionspace_pointcloud(c_fspace)
fset = atlas_fieldset(c_fset)

call to_fieldset(fld, fspace, fset)

end subroutine qg_field_to_fieldset_c

! ------------------------------------------------------------------------------

subroutine qg_field_from_fieldset_c(c_key_fld, c_fset) bind(c,name='qg_field_from_fieldset_f90')
use atlas_module
use iso_c_binding
use qg_fields
implicit none
integer(c_int), intent(in)     :: c_key_fld
type(c_ptr), intent(in), value :: c_fset

type(qg_field), pointer :: fld
type(atlas_fieldset) :: fset

! -- get objects

fld => get_fields(c_key_fld)
fset = atlas_fieldset(c_fset)

call from_fieldset(fld, fset)

end subroutine qg_field_from_fieldset_c
