! (C) Copyright 2009-2016 ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
! In applying this licence, ECMWF does not waive the privileges and immunities 
! granted to it by virtue of its status as an intergovernmental organisation nor
! does it submit to any jurisdiction.

!> Interfaces to be called from C++ for Fortran handling of QG model fields

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_create_c(c_key_self, c_nx, c_ny, c_nl) bind(c,name='qg_generic_ctl_vec_create_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
implicit none
integer(c_int), intent(inout) :: c_key_self

integer(c_int), intent(in) :: c_nx !< nx
integer(c_int), intent(in) :: c_ny !< ny
integer(c_int), intent(in) :: c_nl !< nl

type(qg_generic_ctl_vec)          :: template 
type(qg_generic_ctl_vec), pointer :: self

integer :: nx, ny, nl

! -- get objects

call generic_ctl_vec_list%add(c_key_self,template)
self => get_generic_ctl_vec(c_key_self)

nx = c_nx
ny = c_ny
nl = c_nl

call create(self, nx, ny, nl)

end subroutine qg_generic_ctl_vec_create_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_delete_c(c_key_self) bind(c,name='qg_generic_ctl_vec_delete_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
implicit none
integer(c_int), intent(inout) :: c_key_self
type(qg_generic_ctl_vec), pointer :: self

! -- get objects

self => get_generic_ctl_vec(c_key_self)

call delete(self)

call generic_ctl_vec_list%remove(c_key_self)

end subroutine qg_generic_ctl_vec_delete_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_zero_c(c_key_self) bind(c,name='qg_generic_ctl_vec_zero_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
implicit none
integer(c_int), intent(in) :: c_key_self
type(qg_generic_ctl_vec), pointer :: self

! -- get objects

self => get_generic_ctl_vec(c_key_self)

call zerosctlvec(self)

end subroutine qg_generic_ctl_vec_zero_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_random_c(c_key_self) bind(c,name='qg_generic_ctl_vec_random_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
implicit none
integer(c_int), intent(in) :: c_key_self
type(qg_generic_ctl_vec), pointer :: self

! -- get objects

self => get_generic_ctl_vec(c_key_self)

call random(self)

end subroutine qg_generic_ctl_vec_random_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_copy_c(c_key_self,c_key_rhs) bind(c,name='qg_generic_ctl_vec_copy_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
implicit none
integer(c_int), intent(in) :: c_key_self
integer(c_int), intent(in) :: c_key_rhs

type(qg_generic_ctl_vec), pointer :: self
type(qg_generic_ctl_vec), pointer :: rhs

! -- get objects

self => get_generic_ctl_vec(c_key_self)
rhs => get_generic_ctl_vec(c_key_rhs)

call copy(self, rhs)

end subroutine qg_generic_ctl_vec_copy_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_self_add_c(c_key_self,c_key_rhs) bind(c,name='qg_generic_ctl_vec_self_add_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
implicit none
integer(c_int), intent(in) :: c_key_self
integer(c_int), intent(in) :: c_key_rhs

type(qg_generic_ctl_vec), pointer :: self
type(qg_generic_ctl_vec), pointer :: rhs

! -- get objects

self => get_generic_ctl_vec(c_key_self)
rhs => get_generic_ctl_vec(c_key_rhs)

call self_add(self,rhs)

end subroutine qg_generic_ctl_vec_self_add_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_self_schur_c(c_key_self,c_key_rhs) bind(c,name='qg_generic_ctl_vec_self_schur_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
implicit none
integer(c_int), intent(in) :: c_key_self
integer(c_int), intent(in) :: c_key_rhs

type(qg_generic_ctl_vec), pointer :: self
type(qg_generic_ctl_vec), pointer :: rhs

! -- get objects

self => get_generic_ctl_vec(c_key_self)
rhs => get_generic_ctl_vec(c_key_rhs)

call self_schur(self,rhs)

end subroutine qg_generic_ctl_vec_self_schur_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_self_sub_c(c_key_self,c_key_rhs) bind(c,name='qg_generic_ctl_vec_self_sub_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
implicit none
integer(c_int), intent(in) :: c_key_self
integer(c_int), intent(in) :: c_key_rhs

type(qg_generic_ctl_vec), pointer :: self
type(qg_generic_ctl_vec), pointer :: rhs

! -- get objects

self => get_generic_ctl_vec(c_key_self)
rhs => get_generic_ctl_vec(c_key_rhs)

call self_sub(self,rhs)

end subroutine qg_generic_ctl_vec_self_sub_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_self_mul_c(c_key_self,c_zz) bind(c,name='qg_generic_ctl_vec_self_mul_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
use kinds
implicit none
integer(c_int), intent(in) :: c_key_self
real(c_double), intent(in) :: c_zz
type(qg_generic_ctl_vec), pointer :: self
real(kind=kind_real) :: zz

! -- get objects

self => get_generic_ctl_vec(c_key_self)

zz = c_zz

call self_mul(self,zz)

end subroutine qg_generic_ctl_vec_self_mul_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_axpy_c(c_key_self,c_zz,c_key_rhs) bind(c,name='qg_generic_ctl_vec_axpy_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
use kinds
implicit none
integer(c_int), intent(in) :: c_key_self
real(c_double), intent(in) :: c_zz
integer(c_int), intent(in) :: c_key_rhs

type(qg_generic_ctl_vec), pointer :: self
type(qg_generic_ctl_vec), pointer :: rhs
real(kind=kind_real) :: zz

! -- get objects

self => get_generic_ctl_vec(c_key_self)
rhs => get_generic_ctl_vec(c_key_rhs)

zz = c_zz

call axpy(self,zz,rhs)

end subroutine qg_generic_ctl_vec_axpy_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_dot_prod_c(c_key_fld1,c_key_fld2,c_prod) bind(c,name='qg_generic_ctl_vec_dot_prod_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
use kinds
implicit none
integer(c_int), intent(in)    :: c_key_fld1, c_key_fld2
real(c_double), intent(inout) :: c_prod
real(kind=kind_real) :: zz
type(qg_generic_ctl_vec), pointer :: fld1, fld2

! -- get objects

fld1 => get_generic_ctl_vec(c_key_fld1)
fld2 => get_generic_ctl_vec(c_key_fld2)

call dot_prod(fld1,fld2,zz)

c_prod = zz

end subroutine qg_generic_ctl_vec_dot_prod_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_change_resol_c(c_key_fld,c_key_rhs) bind(c,name='qg_generic_ctl_vec_change_resol_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
implicit none
integer(c_int), intent(in) :: c_key_fld
integer(c_int), intent(in) :: c_key_rhs
type(qg_generic_ctl_vec), pointer :: fld, rhs

! -- get objects

fld => get_generic_ctl_vec(c_key_fld)
rhs => get_generic_ctl_vec(c_key_rhs)

call change_resol(fld,rhs)

end subroutine qg_generic_ctl_vec_change_resol_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_get_section_c(c_key_fld,section,iistart,iiend) bind(c,name='qg_generic_ctl_vec_get_section_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
implicit none
integer(c_int), intent(in)  :: c_key_fld
real(c_double), intent(out) :: section(iiend-iistart+1)
integer(c_int), intent(in)  :: iistart
integer(c_int), intent(in)  :: iiend

type(qg_generic_ctl_vec), pointer :: fld

! -- get objects

fld => get_generic_ctl_vec(c_key_fld)

call get_section(fld,section,iistart,iiend)

end subroutine qg_generic_ctl_vec_get_section_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_set_section_c(c_key_fld,section,iistart,iiend) bind(c,name='qg_generic_ctl_vec_set_section_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
implicit none
integer(c_int), intent(in) :: c_key_fld
real(c_double), intent(in) :: section(iiend-iistart+1)
integer(c_int), intent(in) :: iistart
integer(c_int), intent(in) :: iiend

type(qg_generic_ctl_vec), pointer :: fld

! -- get objects

fld => get_generic_ctl_vec(c_key_fld)

call set_section(fld,section,iistart,iiend)

end subroutine qg_generic_ctl_vec_set_section_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_read_file_c(c_key_fld, c_conf) bind(c,name='qg_generic_ctl_vec_read_file_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod

implicit none
integer(c_int), intent(in) :: c_key_fld  !< Fields
type(c_ptr), intent(in)    :: c_conf !< Configuration

type(qg_generic_ctl_vec), pointer :: fld

! -- get objects

fld => get_generic_ctl_vec(c_key_fld)

call read_file(fld, c_conf)

end subroutine qg_generic_ctl_vec_read_file_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_write_file_c(c_key_fld, c_conf) bind(c,name='qg_generic_ctl_vec_write_file_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod

implicit none
integer(c_int), intent(in) :: c_key_fld  !< Fields
type(c_ptr), intent(in) :: c_conf !< Configuration

type(qg_generic_ctl_vec), pointer :: fld

! -- get objects

fld => get_generic_ctl_vec(c_key_fld)

call write_file(fld, c_conf)

end subroutine qg_generic_ctl_vec_write_file_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_gpnorm_c(c_key_fld, pstat) bind(c,name='qg_generic_ctl_vec_gpnorm_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
use kinds
implicit none
integer(c_int), intent(in) :: c_key_fld
real(c_double), intent(inout) :: pstat(3)

type(qg_generic_ctl_vec), pointer :: fld
real(kind=kind_real) :: zstat(3)
integer :: js

! -- get objects

fld => get_generic_ctl_vec(c_key_fld)

call gpnorm(fld, zstat)
do js = 1, 3
  pstat(js) = zstat(js)
enddo

end subroutine qg_generic_ctl_vec_gpnorm_c

! ------------------------------------------------------------------------------

subroutine qg_generic_ctl_vec_rms_c(c_key_fld, prms) bind(c,name='qg_generic_ctl_vec_rms_f90')
use iso_c_binding
use qg_generic_ctl_vec_mod
use kinds
implicit none
integer(c_int), intent(in) :: c_key_fld
real(c_double), intent(inout) :: prms

type(qg_generic_ctl_vec), pointer :: fld
real(kind=kind_real) :: zz

! -- get objects

fld => get_generic_ctl_vec(c_key_fld)

call fldrms(fld, zz)

prms = zz

end subroutine qg_generic_ctl_vec_rms_c

! ------------------------------------------------------------------------------
