! (C) Copyright 2009-2016 ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
! In applying this licence, ECMWF does not waive the privileges and immunities 
! granted to it by virtue of its status as an intergovernmental organisation nor
! does it submit to any jurisdiction.

!> Fortran module handling geometry for the QG model

module qg_geom_mod

use iso_c_binding
use config_mod
use kinds
use pm_linked_list_mod

implicit none
private
public :: qg_geom, setup, clone, grid_info
public :: geom_list, get_geom

! ------------------------------------------------------------------------------

!> Fortran derived type to hold geometry data for the QG model
type :: qg_geom
  integer :: nx
  integer :: ny
end type qg_geom

!> Global list
type(pm_linked_list) :: geom_list

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

subroutine setup(self,c_conf)
implicit none
type(qg_geom), intent(inout) :: self
type(c_ptr), intent(in)      :: c_conf

self%nx = config_get_int(c_conf, "nx")
self%ny = config_get_int(c_conf, "ny")

end subroutine setup

! ------------------------------------------------------------------------------

subroutine clone(self,other)
implicit none
type(qg_geom), intent(in   ) :: self
type(qg_geom), intent(inout) :: other

other%nx = self%nx
other%ny = self%ny

end subroutine clone

! ------------------------------------------------------------------------------

subroutine grid_info(self,nx,ny)
implicit none
type(qg_geom), intent(in) :: self
integer, intent(inout)    :: nx
integer, intent(inout)    :: ny

nx = self%nx
ny = self%ny

end subroutine grid_info

! ------------------------------------------------------------------------------

function get_geom(c_self)
use iso_c_binding
implicit none
integer(kind=c_int), intent(in) :: c_self
type(qg_geom), pointer :: get_geom
class(*), pointer :: this=>null()

call geom_list%get(c_self, this)

select type(self => this)
type is (qg_geom)
  get_geom=>self
class default
  call abor1_ftn('get_geom : unexpected type ')
end select

end function get_geom

! ------------------------------------------------------------------------------
end module qg_geom_mod
