/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef OOPS_ASSIMILATION_MTMATRIX_H_
#define OOPS_ASSIMILATION_MTMATRIX_H_

#include "eckit/memory/NonCopyable.h"
#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/DualVector.h"
#include "oops/base/PostProcessor.h"
#include "oops/base/PostProcessorAD.h"
#include "oops/interface/Increment.h"

namespace oops {

/// The \f$ M^T \f$ matrix.
/*!
 *  The solvers represent matrices as objects that implement a "multiply"
 *  method. This class defines objects that apply a linear model adjoint
 *  \f$ M^T \f$  matrix which
 */

template <typename MODEL>
class MtMatrix : private eckit::NonCopyable {
  using CtrlInc_ = ControlIncrement<MODEL>;
  using CostFct_ = CostFunction<MODEL>;
  using Increment_ = Increment<MODEL>;

 public:
  explicit MtMatrix(const CostFct_& j) : j_(j) {}

  void multiply(
      ControlIncrement<MODEL>& dx,
      PostProcessor<Increment_> post = PostProcessor<Increment_>()) const {
    PostProcessorAD<Increment_> cost;
    j_.runAD(dx, cost, post, false);
  }

 private:
  CostFct_ const& j_;
};

}  // namespace oops

#endif  // OOPS_ASSIMILATION_MTMATRIX_H_
