/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef OOPS_ASSIMILATION_SYMMATSPECTRUM_H_
#define OOPS_ASSIMILATION_SYMMATSPECTRUM_H_

#include "eckit/maths/Matrix.h"

extern "C" {
void FtnSymMatSpectrum(const int &, const double *, double *, double *);
}

namespace oops {

using eckit::maths::ColVector;
using eckit::maths::Matrix;

void SymMatSpectrum(const Matrix<double> &m_A, ColVector<double> &cv_U,
                    Matrix<double> &m_L) {
  ASSERT(m_A.cols() == m_A.rows());
  const size_t nn = m_A.cols();

  if (nn > 0) {
    cv_U.resize(nn);
    m_L.resize(nn, nn);

    // Call Fortran wrapper to DSYEV
    FtnSymMatSpectrum(nn, m_A.data(), cv_U.data(), m_L.data());
  }
}

}  // namespace oops

#endif  // OOPS_ASSIMILATION_SYMMATSPECTRUM_H_
