/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor
 * does it submit to any jurisdiction.
 */

#ifndef OOPS_BASE_OBSERVERENVARAD_H_
#define OOPS_BASE_OBSERVERENVARAD_H_

#include <memory>
#include <unordered_map>

#include "oops/base/Increment4D.h"
#include "oops/base/PostBaseAD.h"
#include "oops/interface/GenericMatrix.h"
#include "util/DateTime.h"
#include "util/Duration.h"

namespace oops {

/// Computes observation equivalent AD and AD to/from increments.

template <typename MODEL, typename INCR>
class ObserverEnVarAD : public PostBaseAD<INCR> {
  using Increment4D_ = Increment4D<MODEL>;
  using GenericMatrix_ = GenericMatrix<MODEL>;

 public:
  ObserverEnVarAD(const Increment4D_ &, std::shared_ptr<GenericMatrix_>);
  ~ObserverEnVarAD() {}

 private:
  // Methods
  void doFirstAD(INCR &, const util::DateTime &,
                 const util::Duration &) override;
  void doProcessingAD(INCR &) override;
  void doLastAD(INCR &) override;

  // Data
  using Incr4D_ = std::unordered_map<util::DateTime, const INCR *>;
  std::shared_ptr<GenericMatrix_> to_low_resol_;
  Incr4D_ dx_;
};

// ====================================================================================

template <typename MODEL, typename INCR>
ObserverEnVarAD<MODEL, INCR>::ObserverEnVarAD(
    const Increment4D_ &dx, std::shared_ptr<GenericMatrix_> to_low_resol_)
    : PostBaseAD<INCR>(
          dx[dx.first()].validTime(), dx[dx.last()].validTime(),
          dx[dx.first() + 1].validTime() - dx[dx.first()].validTime()),
      to_low_resol_(to_low_resol_) {
  for (int k = dx.first(); k <= dx.last(); ++k) {
    dx_.emplace(dx[k].validTime(), &dx[k]);
  }
}
// -----------------------------------------------------------------------------
template <typename MODEL, typename INCR>
void ObserverEnVarAD<MODEL, INCR>::doFirstAD(INCR &dx,
                                             const util::DateTime &bgn,
                                             const util::Duration &tstep) {}
// -----------------------------------------------------------------------------
template <typename MODEL, typename INCR>
void ObserverEnVarAD<MODEL, INCR>::doProcessingAD(INCR &dx) {
  if (to_low_resol_.get()) {
    INCR dxin(*(dx_[dx.validTime()]));
    to_low_resol_->transposeInverseMultiply(dxin);
    dx.axpy(1.0, dxin);
  } else {
    dx.axpy(1.0, *(dx_[dx.validTime()]));
  }
}
// -----------------------------------------------------------------------------
template <typename MODEL, typename INCR>
void ObserverEnVarAD<MODEL, INCR>::doLastAD(INCR &) {}
// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // OOPS_BASE_OBSERVERENVARAD_H_
