/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef OOPS_BASE_STATEINFO_H_
#define OOPS_BASE_STATEINFO_H_

#include <string>

#include "eckit/config/Configuration.h"
#include "oops/base/PostBase.h"
#include "util/DateTime.h"
#include "util/Logger.h"

namespace oops {

/*! @brief Handles diagnostic prints of forecast fields.
 */

template <typename FLDS>
class StateInfo : public PostBase<FLDS> {
 public:
  /// Constructor
  StateInfo(const eckit::Configuration& conf)
      : PostBase<FLDS>(conf), sgrep_("fc") {}
  /// Constructor
  StateInfo(const std::string sgrep, const eckit::Configuration& conf)
      : PostBase<FLDS>(conf), sgrep_(sgrep) {}
  /// Constructor
  StateInfo(const std::string sgrep, const util::Duration& freq)
      : PostBase<FLDS>(freq), sgrep_(sgrep) {}
  /// Destructor
  ~StateInfo() {}

 private:
  void doProcessing(const FLDS& xx) override {
    /// Print state diagnostics
    Log::info() << sgrep_ << xx << std::endl;
  }

  const std::string sgrep_;
};

}  // namespace oops

#endif  // OOPS_BASE_STATEINFO_H_
