/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef OOPS_BASE_STATESAVER_H_
#define OOPS_BASE_STATESAVER_H_

#include "eckit/config/LocalConfiguration.h"
#include "oops/base/PostBase.h"
#include "util/DateTime.h"

namespace oops {

/*! @brief Store forecast fields.
 *
 *  Store forecast fields for late 4D start.
 */

template <typename FLDS>
class StateSaver : public PostBase<FLDS> {
 public:
  StateSaver(const eckit::Configuration& conf)
      : PostBase<FLDS>(conf), ppConfig_(conf) {}
  ~StateSaver() {}

  std::vector<std::shared_ptr<FLDS> >& releaseFields() { return flds_; }
  const std::vector<std::shared_ptr<FLDS> >& releaseFields() const {
    return flds_;
  }

 private:
  const eckit::LocalConfiguration ppConfig_;
  std::vector<std::shared_ptr<FLDS> > flds_;
  void doProcessing(const FLDS& xx) override {
    flds_.push_back(std::shared_ptr<FLDS>(new FLDS(xx)));
  }
};

}  // namespace oops

#endif  // OOPS_BASE_STATESAVER_H_
