/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef OOPS_BASE_STATEWRITER_H_
#define OOPS_BASE_STATEWRITER_H_

#include "eckit/config/LocalConfiguration.h"
#include "eckit/exception/Exceptions.h"
#include "oops/base/PostBase.h"
#include "util/DateTime.h"

namespace oops {

/*! @brief Handles writing-out of forecast fields.
 */

template <typename FLDS>
class StateWriter : public PostBase<FLDS> {
 public:
  /// Constructor
  StateWriter(const eckit::Configuration& conf)
      : PostBase<FLDS>(conf), ppConfig_(conf) {}
  /// Constructor
  StateWriter(const util::DateTime& start, const eckit::Configuration& conf)
      : PostBase<FLDS>(start, conf), ppConfig_(conf) {}
  /// Destructor
  ~StateWriter() {}

 private:
  void doProcessing(const FLDS& xx) override {
    eckit::LocalConfiguration config(ppConfig_);
    if (config.has("appendStepToName")) {
      ASSERT(config.has("restart_file"));
      std::string restartFile = config.getString("restart_file");
      restartFile += xx.validTime().toString();
      config.set("restart_file", restartFile);
    }
    /// Write the state
    xx.write(config);
  }

  const eckit::LocalConfiguration ppConfig_;
};

}  // namespace oops

#endif  // OOPS_BASE_STATEWRITER_H_
