/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef OOPS_BASE_INSTANTIATECOVARFACTORY_H_
#define OOPS_BASE_INSTANTIATECOVARFACTORY_H_

#include "oops/base/EnsembleCovariance.h"
#include "oops/base/EnsembleCovariance4D.h"
#include "oops/base/EnsembleCovariance4DDistributed.h"
#include "oops/base/EnsembleCovariance4DScaleDependent.h"
#include "oops/base/EnsembleCovariance4DWithChangeOfVariable.h"
#include "oops/base/EnsembleCovsInterpol.h"
#include "oops/base/EnsembleCovsInterpol4D.h"
#include "oops/base/HybridCovariance.h"
#include "oops/base/HybridCovariance4D.h"
#include "oops/base/HybridCovarianceGeneralCoeff.h"
#include "oops/base/ModelSpaceCovarianceBase.h"
#include "oops/base/StaticCovariance4D.h"
#include "oops/base/StaticWithTLModelCovariance4D.h"
#include "oops/interface/ErrorCovariance.h"

namespace oops {

template <typename MODEL>
void instantiateCovarFactory() {
  static CovarMaker<MODEL, HybridCovariance<MODEL> > makerHybrid_("hybrid");
  static CovarMaker<MODEL, HybridCovarianceGeneralCoeff<MODEL> >
      makerHybridGen_("hybrid_general_coeff");
  static CovarMaker<MODEL, EnsembleCovariance<MODEL> > makerEnsemble_(
      "ensemble");
  static CovarMaker<MODEL, ErrorCovariance<MODEL> > makerModel_("static");
  static CovarMaker<MODEL, EnsembleCovsInterpol<MODEL> > makerEnsCovsInterpol_(
      "ens_interpol");
  static Covar4DMaker<MODEL, StaticCovariance4D<MODEL> > makerStatic4D_(
      "static4D");
  static Covar4DMaker<MODEL, HybridCovariance4D<MODEL> > makerHybrid4D_(
      "hybrid");
  static Covar4DMaker<MODEL, EnsembleCovariance4D<MODEL> > makerEnsemble4D_(
      "ensemble");
  static Covar4DMaker<MODEL, EnsembleCovariance4DWithChangeOfVariable<MODEL> >
      makerEnsemble4DPsiKhi_("ensemble_with_change_of_variable");
  static Covar4DMaker<MODEL, StaticWithTLModelCovariance4D<MODEL> >
      makerStaticTL4D_("static_with_tlmodel");
  static Covar4DMaker<MODEL, EnsembleCovsInterpol4D<MODEL> >
      makerEnsembleInterp4D_("ens_interpol");
  static Covar4DMaker<MODEL, EnsembleCovariance4DScaleDependent<MODEL> >
      makerEnsembleScaleDep4D_("ensemble_scale_dependent");
  static Covar4DMaker<MODEL, EnsembleCovariance4DDistributed<MODEL> >
      makerEnsembleDist4D_("ensemble_distributed");
}

}  // namespace oops

#endif  // OOPS_BASE_INSTANTIATECOVARFACTORY_H_
