/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef OOPS_GENERIC_GENERICCTLVEC_H_
#define OOPS_GENERIC_GENERICCTLVEC_H_

#include <memory>
#include <random>
#include <string>

#include "atlas/field.h"

#include "util/ObjectCounter.h"
#include "util/Printable.h"

namespace oops {

// -----------------------------------------------------------------------------

/*! @brief Generic control vector class
 */

class GenericCtlVec : public util::Printable,
                      private util::ObjectCounter<GenericCtlVec> {
 public:
  static const std::string classname() { return "oops::GenericCtlVec"; }

  /// Constructors
  GenericCtlVec() {}
  explicit GenericCtlVec(const size_t &);

  /// Copy constructor
  GenericCtlVec(const GenericCtlVec &, const bool copy = true);

  /// Destructor
  virtual ~GenericCtlVec();

  // Linear algebra operators
  void zero();
  GenericCtlVec &operator=(const GenericCtlVec &);
  GenericCtlVec &operator+=(const GenericCtlVec &);
  GenericCtlVec &operator-=(const GenericCtlVec &);
  GenericCtlVec &operator*=(const double &);
  void axpy(const double &, const GenericCtlVec &, const bool check = true);
  double local_dot_product_with(const GenericCtlVec &) const;
  void schur_product_with(const GenericCtlVec &);
  void random();

  // Section (required for re-orthogonalization in LMP)
  /// Get section of the control vector
  void getSection(std::vector<double> &, const int &, const int &) const;
  /// Set section of the control vector
  void setSection(const std::vector<double> &, const int &, const int &);

  // I/O and diagnostics
  /// Read control vector
  void read(const eckit::Configuration &);
  /// Write control vector
  void write(const eckit::Configuration &) const;
  /// L-2 norm
  double norm() const;

  /// Size accessor
  size_t size() const { return n_; }

  /// Data accessor
  const atlas::Field &data() const { return data_; }
  atlas::Field &data() { return data_; }

 private:
  void print(std::ostream &) const;

  size_t n_;
  size_t nGlb_;
  atlas::Field data_;
  std::vector<int> sendcounts_;
  std::vector<int> displs_;
};

}  // namespace oops

#endif  // OOPS_GENERIC_GENERICCTLVEC_H_
