/*
 * Copyright 2011 ECMWF
 *
 * This software was developed at ECMWF for evaluation
 * and may be used for academic and research purposes only.
 * The software is provided as is without any warranty.
 *
 * This software can be used, copied and modified but not
 * redistributed or sold. This notice must be reproduced
 * on each copy made.
 */

#ifndef OOPS_BASE_POSTPROCESSORMODEL_H_
#define OOPS_BASE_POSTPROCESSORMODEL_H_

#include <memory>

#include "oops/base/PostBase.h"
#include "oops/interface/Geometry.h"
#include "util/DateTime.h"
#include "util/Duration.h"

namespace oops {

// -----------------------------------------------------------------------------

/// Compute time average of states or increments during model run.
/*!
 *  Derived classes will compute IAU
 */

template <typename MODEL, typename STATE>
class PostProcessorModel : public PostBase<STATE> {
  using PostProcessor_ = typename MODEL::PostProcessor;

 public:
  PostProcessorModel(const eckit::Configuration &);
  virtual ~PostProcessorModel() {}

 private:
  void doInitialize(const STATE &, const util::DateTime &,
                    const util::Duration &);
  void doProcessing(const STATE &);

  std::unique_ptr<PostProcessor_> post_processor_;
  std::unique_ptr<const util::DateTime> start_;
  int tstep_;
  const eckit::LocalConfiguration config_;
};

// =============================================================================

template <typename MODEL, typename STATE>
PostProcessorModel<MODEL, STATE>::PostProcessorModel(
    const eckit::Configuration &config)
    : config_(config), PostBase<STATE>(config) {
  post_processor_.reset(new PostProcessor_(config));
}

// -----------------------------------------------------------------------------

template <typename MODEL, typename STATE>
void PostProcessorModel<MODEL, STATE>::doInitialize(
    const STATE &xx, const util::DateTime &end, const util::Duration &tstep) {
  start_.reset(new util::DateTime(xx.validTime()));
  tstep_ = tstep.toSeconds();
  post_processor_->initialize(xx.state());
}

// -----------------------------------------------------------------------------

template <typename MODEL, typename STATE>
void PostProcessorModel<MODEL, STATE>::doProcessing(const STATE &xx) {
  util::Duration step(xx.validTime() - (*start_));
  int jstep = step.toSeconds();

  post_processor_->process(xx.state(), jstep);
}

// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // OOPS_BASE_POSTPROCESSORMODEL_H_
