/*
 * Copyright 2011 ECMWF
 *
 * This software was developed at ECMWF for evaluation
 * and may be used for academic and research purposes only.
 * The software is provided as is without any warranty.
 *
 * This software can be used, copied and modified but not
 * redistributed or sold. This notice must be reproduced
 * on each copy made.
 */

#ifndef OOPS_RUNS_ASSIM_ENSEMBLE_H_
#define OOPS_RUNS_ASSIM_ENSEMBLE_H_

#include <string>

#include "eckit/config/LocalConfiguration.h"
#include "oops/runs/Application.h"
#include "oops/runs/Variational.h"
#include "util/DateTime.h"
#include "util/Duration.h"
#include "util/Logger.h"
#include "util/MPIWrapper.h"

namespace oops {

template <typename MODEL>
class AssimEnsemble : public Application {
 public:
  // -----------------------------------------------------------------------------
  AssimEnsemble() {}

  // -----------------------------------------------------------------------------
  virtual ~AssimEnsemble() {}

  // -----------------------------------------------------------------------------
  int execute(const eckit::Configuration& fullConfig) const {
    util::MPIWrapper& mpiw = util::MPIWrapper::Instance();
    Variational<MODEL> var;

    std::vector<eckit::LocalConfiguration> assimConfig;
    fullConfig.get("assim_member", assimConfig);

    unsigned nb_assim = assimConfig.size();
    unsigned iassim = mpiw.my_rank();
    ASSERT(iassim < nb_assim);

    Log::info() << "Performing minim for member:" << iassim << " out of "
                << nb_assim;

    return var.execute(assimConfig[iassim]);
  }

  // -----------------------------------------------------------------------------
 private:
  std::string appname() const {
    return "oops::AssimEnsemble<" + MODEL::name() + ">";
  }
  // -----------------------------------------------------------------------------
};

}  // namespace oops
#endif  // OOPS_RUNS_ASSIM_ENSEMBLE_H_
