/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef OOPS_RUNS_MODELAUXNN_H_
#define OOPS_RUNS_MODELAUXNN_H_

#include <string>

#include "eckit/config/LocalConfiguration.h"
#include "oops/base/PostProcessor.h"
#include "oops/base/StateInfo.h"
#include "oops/base/StateWriter.h"
#include "oops/interface/Geometry.h"
#include "oops/interface/Model.h"
#include "oops/interface/ModelAuxControl.h"
#include "oops/interface/ModelAuxControlEstimator.h"
#include "oops/interface/State.h"
#include "oops/runs/Application.h"
#include "util/DateTime.h"
#include "util/Duration.h"
#include "util/Logger.h"

namespace oops {

template <typename MODEL>
class ModelAuxNN : public Application {
  using Geometry_ = Geometry<MODEL>;
  using Model_ = Model<MODEL>;
  using ModelAux_ = ModelAuxControl<MODEL>;
  using ModelAuxEstimator_ = ModelAuxControlEstimator<MODEL>;
  using State_ = State<MODEL>;

 public:
  // -----------------------------------------------------------------------------
  ModelAuxNN() {}
  // -----------------------------------------------------------------------------
  virtual ~ModelAuxNN() {}
  // -----------------------------------------------------------------------------
  int execute(const eckit::Configuration& fullConfig) const {
    //  Setup resolution
    const eckit::LocalConfiguration resolConfig(fullConfig, "resolution");
    const Geometry_ resol(resolConfig);

    //  Setup Model
    const eckit::LocalConfiguration modelConfig(fullConfig, "model");
    const Model_ model(resol, modelConfig);

    //  Setup background state
    const eckit::LocalConfiguration initialConfig(fullConfig, "initial");
    const eckit::LocalConfiguration xbConfig(initialConfig, "xb");
    State_ xb(resol, model, xbConfig);
    Log::info() << "Background state: " << xb.norm() << std::endl;

    //  Setup an state
    const eckit::LocalConfiguration anConfig(initialConfig, "an");
    State_ an(resol, model, anConfig);
    Log::info() << "Previous Analysis state: " << an.norm() << std::endl;

    //  Setup model error estimator
    const ModelAuxEstimator_ moderrEstimator(resol, model, initialConfig);

    //  Setup augmented state
    ModelAux_ moderr(resol, model, initialConfig);

    //  Estimate model error
    moderrEstimator.estimate(xb, an, moderr);

    //  Compute the model error norm
    Log::test() << "ModelAuxControl state: " << moderr.norm() << std::endl;

    return 0;
  }
  // -----------------------------------------------------------------------------
 private:
  std::string appname() const {
    return "oops::ModelAuxNN<" + MODEL::name() + ">";
  }
  // -----------------------------------------------------------------------------
};

}  // namespace oops
#endif  // OOPS_RUNS_MODELAUXNN_H_
