/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef TEST_INTERFACE_OBSERRORCOVARIANCE_H_
#define TEST_INTERFACE_OBSERRORCOVARIANCE_H_

#include <memory>  // for std::unique_ptr
#include <string>
#include <vector>

#define ECKIT_TESTING_SELF_REGISTER_CASES 0

#include "eckit/config/LocalConfiguration.h"
#include "eckit/memory/NonCopyable.h"
#include "eckit/testing/Test.h"

#include "oops/generic/instantiateObsErrorFactory.h"
#include "oops/interface/Geometry.h"
#include "oops/interface/ObsErrorCovariance.h"
#include "oops/interface/ObservationSpace.h"
#include "oops/runs/Test.h"

#include "test/TestEnvironment.h"

namespace test {

// -----------------------------------------------------------------------------
template <typename MODEL>
class ObsErrorCovarianceFixture : private eckit::NonCopyable {
  using Geometry_ = oops::Geometry<MODEL>;
  using ObsSpace_ = oops::ObservationSpace<MODEL>;

 public:
  static const eckit::Configuration& config() { return *getInstance().conf_; }
  static const ObsSpace_& obspace() { return *getInstance().obspace_; }

 private:
  static ObsErrorCovarianceFixture<MODEL>& getInstance() {
    static ObsErrorCovarianceFixture<MODEL> theObsErrorCovarianceFixture;
    return theObsErrorCovarianceFixture;
  }

  ObsErrorCovarianceFixture() {
    oops::instantiateObsErrorFactory<MODEL>();

    const util::DateTime tbgn(
        TestEnvironment::config().getString("window_begin"));
    const util::DateTime tend(
        TestEnvironment::config().getString("window_end"));

    const eckit::LocalConfiguration geomConfig(TestEnvironment::config(),
                                               "Geometry");
    geom_.reset(new Geometry_(geomConfig));

    std::vector<eckit::LocalConfiguration> obsConfs;
    TestEnvironment::config().get("Observations", obsConfs);
    EXPECT(obsConfs.size() > 0);
    const eckit::LocalConfiguration obsConf(obsConfs[0], "Observation");
    obspace_.reset(new ObsSpace_(obsConf, *geom_, tbgn, tend));

    conf_.reset(new eckit::LocalConfiguration(obsConfs[0], "Covariance"));
  }

  ~ObsErrorCovarianceFixture() {}

  std::unique_ptr<const eckit::LocalConfiguration> conf_;
  std::unique_ptr<Geometry_> geom_;
  std::unique_ptr<const ObsSpace_> obspace_;
};
// -----------------------------------------------------------------------------

template <typename MODEL>
void testConstructor() {
  using Test_ = ObsErrorCovarianceFixture<MODEL>;
  using Covar_ = oops::ObsErrorCovariance<MODEL>;

  std::unique_ptr<Covar_> R(new Covar_(Test_::obspace(), Test_::config()));
  EXPECT(R.get());

  R.reset();
  EXPECT(!R.get());
}

// -----------------------------------------------------------------------------

template <typename MODEL>
class ObsErrorCovariance : public oops::Test {
 public:
  ObsErrorCovariance() {}
  virtual ~ObsErrorCovariance() {}

 private:
  std::string testid() const {
    return "test::ObsErrorCovariance<" + MODEL::name() + ">";
  }

  void register_tests() const {
    std::vector<eckit::testing::Test>& ts = eckit::testing::specification();

    ts.emplace_back(CASE("interface/ObsErrorCovariance/testConstructor") {
      testConstructor<MODEL>();
    });
  }
};

// =============================================================================

}  // namespace test

#endif  // TEST_INTERFACE_OBSERRORCOVARIANCE_H_
