/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef TEST_INTERFACE_VARIABLES_H_
#define TEST_INTERFACE_VARIABLES_H_

#include <memory>  // for std::unique_ptr
#include <string>
#include <vector>

#define ECKIT_TESTING_SELF_REGISTER_CASES 0

#include "eckit/config/Configuration.h"
#include "eckit/memory/NonCopyable.h"
#include "eckit/testing/Test.h"

#include "oops/interface/Variables.h"
#include "oops/runs/Test.h"

#include "test/TestEnvironment.h"

namespace test {

// -----------------------------------------------------------------------------
template <typename MODEL>
class VariablesFixture : private eckit::NonCopyable {
 public:
  static const eckit::Configuration& getConfig() {
    return *getInstance().conf_;
  }

 private:
  static VariablesFixture<MODEL>& getInstance() {
    static VariablesFixture<MODEL> theVariablesFixture;
    return theVariablesFixture;
  }

  VariablesFixture() {
    conf_.reset(
        new eckit::LocalConfiguration(TestEnvironment::config(), "Variables"));
  }

  ~VariablesFixture() {}

  std::unique_ptr<const eckit::LocalConfiguration> conf_;
};
// -----------------------------------------------------------------------------

template <typename MODEL>
void testConstructor() {
  using Variables_ = oops::Variables<MODEL>;

  std::unique_ptr<Variables_> vars(
      new Variables_(VariablesFixture<MODEL>::getConfig()));
  EXPECT(vars.get());

  vars.reset();
  EXPECT(!vars.get());
}

// -----------------------------------------------------------------------------

template <typename MODEL>
void testCopyConstructor() {
  using Variables_ = oops::Variables<MODEL>;
  std::unique_ptr<Variables_> vars(
      new Variables_(VariablesFixture<MODEL>::getConfig()));

  std::unique_ptr<Variables_> other(new Variables_(*vars));
  EXPECT(other.get());

  other.reset();
  EXPECT(!other.get());

  EXPECT(vars.get());
}

// -----------------------------------------------------------------------------

template <typename MODEL>
class Variables : public oops::Test {
 public:
  Variables() {}
  virtual ~Variables() {}

 private:
  std::string testid() const {
    return "test::Variables<" + MODEL::name() + ">";
  }

  void register_tests() const {
    std::vector<eckit::testing::Test>& ts = eckit::testing::specification();

    ts.emplace_back(CASE("interface/Variables/testConstructor") {
      testConstructor<MODEL>();
    });
    ts.emplace_back(CASE("interface/Variables/testCopyConstructor") {
      testCopyConstructor<MODEL>();
    });
  }
};

// =============================================================================

}  // namespace test

#endif  // TEST_INTERFACE_VARIABLES_H_
