/*
 * Copyright 2011 ECMWF
 *
 * This software was developed at ECMWF for evaluation
 * and may be used for academic and research purposes only.
 * The software is provided as is without any warranty.
 *
 * This software can be used, copied and modified but not
 * redistributed or sold. This notice must be reproduced
 * on each copy made.
 */

#ifndef OOPS_TEST_SUITE_CHANGE_RESOLUTION_H_
#define OOPS_TEST_SUITE_CHANGE_RESOLUTION_H_

#include <iostream>

#include "util/Logger.h"
#include "util/Duration.h"
#include "util/dot_product.h"

namespace test {

// -----------------------------------------------------------------------------
TEST_CASE(mix,test_fields_change_resolution) {
  TEST_MESSAGE(
      "Test fields change of resolution  ");
  std::ostringstream oss;
  State_ x0(state());

  // Setup resolution
  const Config_ resolConfig(getConfig(), "resolution_test");
  Geometry_ resol_test(resolConfig);

  // Change to second resolution and then back to the first
  State_ x1(resol_test, x0);
  State_ x2(getResolution(), x1);

  Increment_ dx(increment());
  dx.diff(x0,x2);
  double dxdx = dot_product(dx, dx);
  oss << dxdx;

  // Report result
  oss << std::setprecision(20) << "||x0-x2|| = " << dxdx;
  TEST_MESSAGE(oss.str());
  checkResult(dxdx, "expected_diff");
}

}

#endif
