/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef UTIL_LOGBOOKWRITER_H_
#define UTIL_LOGBOOKWRITER_H_

#include <string>

#include "eckit/memory/NonCopyable.h"
#include "util/LogbookHelper.h"

namespace util {

// -----------------------------------------------------------------------------

/*! @brief LogbookWriter

 *   LogbookWriter class allows to record a (key,value) pair in the Logbook
 *  (held by the LogbookHelper); this can be used to infer the state of the
 *  execution of oops from the Model
 */

template <typename ValueT>
class LogbookWriter : private eckit::NonCopyable {
 public:
  /// Contructor, destructor
  LogbookWriter(const std::string &, const ValueT &);
  ~LogbookWriter();

 private:
  const std::string key_;
};

// -----------------------------------------------------------------------------

template <typename ValueT>
LogbookWriter<ValueT>::LogbookWriter(const std::string &key, const ValueT &val)
    : key_(key) {
  LogbookHelper::startRecording<ValueT>(key_, val);
}

// -----------------------------------------------------------------------------

template <typename ValueT>
LogbookWriter<ValueT>::~LogbookWriter() {
  LogbookHelper::finishRecording(key_);
};

// -----------------------------------------------------------------------------

}  // namespace util

#endif  // UTIL_LOGBOOKWRITER_H_
