/*
 * Copyright 2011 ECMWF
 *
 * This software was developed at ECMWF for evaluation
 * and may be used for academic and research purposes only.
 * The software is provided as is without any warranty.
 *
 * This software can be used, copied and modified but not
 * redistributed or sold. This notice must be reproduced
 * on each copy made.
 *
 * This class handles MPI communicators for OOPS
 */
#ifndef UTIL_MPI_WRAPPER_H_
#define UTIL_MPI_WRAPPER_H_

#include <memory>
#include <string>

#include "eckit/config/LocalConfiguration.h"
#include "eckit/exception/Exceptions.h"
#include "eckit/mpi/Comm.h"

namespace util {

/// The singleton MPIWrapper handles MPI multiple communicators

class MPIWrapper {
 public:
  void setup(const eckit::Configuration&);

  unsigned int my_rank() const { return my_rank_; }
  unsigned int nb_worlds() const { return nb_worlds_; }
  unsigned int nb_threads() const { return nb_threads_; }

  int model_communicator() const { return comm_in_ftn_; }
  unsigned int my_model_rank() const { return my_model_rank_; }

  const eckit::mpi::Comm& comm() const {
    return eckit::mpi::comm(oops_comm_[0].c_str());
  }

  const eckit::mpi::Comm& comm(unsigned ithread) const {
    ASSERT(ithread < nb_threads_);
    return eckit::mpi::comm(oops_comm_[ithread].c_str());
  }

  double finalize_dot_product(double) const;

  static MPIWrapper& Instance();

 private:
  // Singleton implementation
  static MPIWrapper m_instance_;
  MPIWrapper();
  ~MPIWrapper();

  // Private attributes
  std::string model_comm_;
  std::vector<std::string> oops_comm_;

  int comm_in_ftn_;

  unsigned int nb_threads_, nb_worlds_, my_model_rank_, my_rank_;
};

}  // namespace util

#endif  // UTIL_MPI_WRAPPER_H_
