/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation
 * nor does it submit to any jurisdiction.
 */

#ifndef UTIL_PRINTADJTEST_H_
#define UTIL_PRINTADJTEST_H_

/// Wraps printing out of the result of an adjoint test.

#include <iomanip>
#include <iostream>
#include <string>

#include "util/Printable.h"
#include "util/formats.h"

namespace util {

// -----------------------------------------------------------------------------

class PrintAdjTest : public Printable {
 public:
  PrintAdjTest(const double& dp1, const double& dp2, const std::string& op);
  PrintAdjTest(const double& dp1, const double& dp2, const std::string& op,
               const std::string& opfwd1, const std::string& opfwd2,
               const std::string& opbwd1, const std::string& opbwd2);
  virtual ~PrintAdjTest() {}

 private:
  void print(std::ostream&) const;

  const double& dp1_;
  const double& dp2_;
  const std::string& op_;
  const std::string opfwd1_;
  const std::string opfwd2_;
  const std::string opbwd1_;
  const std::string opbwd2_;
};

// -----------------------------------------------------------------------------

PrintAdjTest::PrintAdjTest(const double& dp1, const double& dp2,
                           const std::string& op)
    : dp1_(dp1),
      dp2_(dp2),
      op_(op),
      opfwd1_(op + " dx1"),
      opfwd2_("dx2"),
      opbwd1_("dx1"),
      opbwd2_(op + "^T" + " dx2") {}

// -----------------------------------------------------------------------------

PrintAdjTest::PrintAdjTest(const double& dp1, const double& dp2,
                           const std::string& op, const std::string& opfwd1,
                           const std::string& opfwd2, const std::string& opbwd1,
                           const std::string& opbwd2)
    : dp1_(dp1),
      dp2_(dp2),
      op_(op),
      opfwd1_(opfwd1),
      opfwd2_(opfwd2),
      opbwd1_(opbwd1),
      opbwd2_(opbwd2) {}

// -----------------------------------------------------------------------------

void PrintAdjTest::print(std::ostream& os) const {
  size_t width = 48;
  const double eps = std::numeric_limits<double>::epsilon();
  const double dpdiff = std::abs(dp1_ - dp2_);
  const double dpdiff_norm = dp1_ == 0.0 ? 0.0 : std::abs(dpdiff / dp1_);
  const double dpdiff_norm_eps = dp1_ == 0.0 ? 0.0 : dpdiff_norm / eps;
  const std::string header("  " + op_ + " adjoint test: ");
  const std::string line_1(header + "<" + opfwd1_ + ", " + opfwd2_ + ">");
  const std::string line_2(header + "<" + opbwd1_ + ", " + opbwd2_ + ">");
  const std::string line_3(header + "difference");
  const std::string line_4(header + "normalized difference");
  const std::string line_5(header + "normalized difference/machine_epsilon");
  os << std::left << std::setw(width) << line_1 << " : " << full_precision(dp1_)
     << std::endl
     << std::setw(width) << line_2 << " : " << full_precision(dp2_) << std::endl
     << std::setw(width) << line_3 << " : " << full_precision(dpdiff)
     << std::endl
     << std::setw(width) << line_4 << " : " << full_precision(dpdiff_norm)
     << std::endl
     << std::setw(width) << line_5 << " : " << dpdiff_norm_eps << std::endl;
}

// =============================================================================

// -----------------------------------------------------------------------------

}  // namespace util

#endif  // UTIL_PRINTADJTEST_H_
